
<?php

/**
 * @file
 * Contains support form for miniOrange SAML Login Module.
 */
include "includes\miniorange_saml_enum.php";

define( "Tab_Class_Names", serialize( array(
    "Identity_Provider" => 'mo_options_enum_identity_provider',
    "Service_Provider"  => 'mo_options_enum_service_provider',
    "Proxy_Setup"       => 'mo_options_enum_proxy_setup'
) ) );
/**
 * Showing Support form info.
 */
function miniorange_saml_export_config($form, &$form_state) {
	
	$form['markup_msg'] = array(
        '#markup' => '<p>If you are looking for an Identity Provider, you can try out <a href="https://idp.miniorange.com" target="_blank">miniOrange On-Premise IdP</a>.</p>'		
    );
	
	if (variable_get('miniorange_saml_customer_admin_email', NULL) == NULL|| variable_get('miniorange_saml_customer_id', NULL) == NULL
    || variable_get('miniorange_saml_customer_admin_token', NULL) == NULL || variable_get('miniorange_saml_customer_api_key', NULL) == NULL) {
    $form['header'] = array(
        '#markup' => '<center><h3>You need to register with miniOrange before using this module.</h3></center>',
    );
    
    return $form;
    }
	
	
    $form['markup_top'] = array(
        '#markup' => '<h3>Export Configuration</h3>' .'<hr/>' .'<br/>'. '<p>This tab will help you to transfer your plugin configurations when you change your Drupal instance.</p>'
            . '<p>Download plugin configuration file by clicking on the button given below and send us this file along with your support query. </p>'		
    );
	
	$form['miniorange_saml_idp_export'] = array(
        '#type' => 'submit',
        '#value' => t('Download Plugin Configuration'),
	    '#prefix'=>'<td>',
        '#suffix'=>'</td>',
	    '#weight'=>1,
        '#submit' => array('miniorange_import_export'),
    );

    return $form;
}

function miniorange_import_export() {

    $tab_class_name      = unserialize( Tab_Class_Names );
    $configuration_array = array();
    foreach ( $tab_class_name as $key => $value ) {
        $configuration_array[ $key ] = mo_get_configuration_array( $value );
    }

    $configuration_array["Version_dependencies"]=mo_get_version_informations();

    header( "Content-Disposition: attachment; filename=miniorange-saml-config.json" );
    echo( json_encode( $configuration_array, JSON_PRETTY_PRINT|JSON_UNESCAPED_SLASHES) );
    exit;
}

function mo_get_configuration_array( $class_name ) {
    $class_object = call_user_func( $class_name . '::getConstants' );
    $mo_array = array();

    foreach ( $class_object as $key => $value ) {

        $mo_option_exists=variable_get($value);
        if($mo_option_exists){
            if(@unserialize($mo_option_exists)!==false){
                $mo_option_exists = unserialize($mo_option_exists);
            }
            $mo_array[ $key ] = $mo_option_exists;
        }
    }
    return $mo_array;
}

function mo_update_configuration_array( $configuration_array ) {
    $tab_class_name = unserialize( Tab_Class_Names );
    foreach ( $tab_class_name as $tab_name => $class_name ) {
        foreach ( $configuration_array[ $tab_name ] as $key => $value ) {
            $option_string = constant( "$class_name::$key" );
            $mo_option_exists = variable_get($option_string);
            if ( $mo_option_exists) {
                if(is_array($value))
                    $value = serialize($value);
                vaiable_set( $option_string, $value );
            }
        }
    }
}

function mo_get_version_informations(){
    $array_version = array();
    $array_version["PHP_version"] = phpversion();
    $array_version["Drupal_version"] = VERSION;
    $array_version["OPEN_SSL"] = mo_saml_is_openssl_installed();
    $array_version["CURL"] = mo_saml_is_curl_installed();
    $array_version["ICONV"] = mo_saml_is_iconv_installed();
    $array_version["DOM"] = mo_saml_is_dom_installed();

    return $array_version;
}

function mo_saml_is_openssl_installed() {
    if ( in_array( 'openssl', get_loaded_extensions() ) ) {
        return 1;
    } else {
        return 0;
    }
}

function mo_saml_is_curl_installed() {
    if ( in_array( 'curl', get_loaded_extensions() ) ) {
        return 1;
    } else {
        return 0;
    }
}

function mo_saml_is_iconv_installed(){

    if ( in_array( 'iconv', get_loaded_extensions() ) ) {
        return 1;
    } else {
        return 0;
    }
}

function mo_saml_is_dom_installed(){

    if ( in_array( 'dom', get_loaded_extensions() ) ) {
        return 1;
    } else {
        return 0;
    }
}




